"""
Proxify

Author: Pierre Dubertrand
Name-US: Proxify
Version: 1.0
Description-US: Creates a Corona Proxy setup in one go.
Note: Saves the proxy file next to the document file, or on the user desktop if the document is not saved

Written for Maxon Cinema 4D 2024.5.1
Python version 3.11

Change log:
1.0.0 ( 2024.09.24) - Initial release
"""

import c4d
from pathlib import Path

doc = c4d.documents.GetActiveDocument()

# 1036396  =  Corona Proxy (*.cgeo)
# 1056572  =  Corona Standalone (*.cdo)
# 1062891  =  Corona VrScene (*.vrscene)
# 1035544 = Corona Proxy Object


def GetAllNodesRecursively(node):
    """Yields all descendants of ``node`` in a recursive fashion.
    """
    if node is None:
        return

    yield node

    for child in node.GetChildren():
        # The recursion is happening here. For every node in the tree, the
        # function is calling itself, maxing out the recursion limit rather
        # quickly.
        for descendant in GetAllNodesRecursively(child):
            yield descendant


def create_cproxy(doc, obj, path):
    """Create a Corona Proxy object

    Args:
        doc (c4d.documents.BaseDocument): The active document
        obj (c4d.BaseObject): The source object giving it's name to the proxy
        path (str): The path to the .cgeo file

    Returns:
        c4d.BaseObject: The Corona Proxy object
    """
    cproxy = c4d.BaseObject(1035544)
    cproxy[c4d.ID_BASELIST_NAME] = obj.GetName()
    cproxy[c4d.CORONA_PROXY_FILENAME] = path
    # cproxy[c4d.CORONA_PROXY_VISUALIZATION_METHOD] = c4d.CORONA_PROXY_VISUALIZATION_FULL_MESH
    return cproxy


def export_selection_as_cproxy(doc, obj, path):
    """Export the current selection as a .cgeo file

    Args:
        doc (c4d.documents.BaseDocument): The active document
        obj (c4d.BaseObject): The source object for the proxy
        path (str): The path to save the proxy file

    Returns:
        str: The path to the .cgeo file
    """

    if obj:

        # Access the settings
        plug = c4d.plugins.FindPlugin(1036396, c4d.PLUGINTYPE_SCENESAVER)
        op = {}
        if plug.Message(c4d.MSG_RETRIEVEPRIVATEDATA, op):
            print(op)
        
        cproxy_exporter = op["imexporter"]
        if cproxy_exporter is None:
            return
        
        # Force settings to get only the current selection
        cproxy_exporter[c4d.CORONA_PROXYEXPORT_ACTIVE_ONLY] = True
        cproxy_exporter[c4d.CORONA_PROXYEXPORT_INCLUDE_CHILDREN] = True

        # Export
        export = c4d.documents.SaveDocument(doc, path, c4d.SAVEDOCUMENTFLAGS_DONTADDTORECENTLIST, 1036396)

        if export == True:
            return path
        else:
            return None


def extract_tags(doc, obj, tag_type = None):
    """Iterate over an object's tags starting from the end, 
    eventually looking for a specific type

    Args:
        doc (c4d.documents.BaseDocument): The active document
        obj (c4d.BaseObject): The object to iterate over on
        tag_type (c4d.BaseTag, optional): The tag type to filter. Defaults to None.

    Returns:
        bool: True

    Yields:
        c4d.BaseTag: A tag
    """
    tag = obj.GetLastTag()
    while tag:
        if tag_type != None:
            if isinstance(tag, tag_type):
                yield tag
        else:
            yield tag
        tag = tag.GetPred()
    return True


def main():

    # Get the selection
    obj = doc.GetActiveObject()

    if obj:

        # Get the scene path
        doc_path = doc.GetDocumentPath()
        proxy_file = obj.GetName() + ".cgeo"

        # Construct a new path from the selected object
        if doc_path == "":
            proxy_path = Path.home() / 'Desktop' / proxy_file
        else:    
            proxy_path = Path(doc_path)
            proxy_path =  proxy_path / proxy_file
        
        proxy_path = str(proxy_path)

        c4d.StopAllThreads()
        
        c4d.StatusSetSpin()

        c4d.StatusSetText('Exporting Corona Proxy...') 

        doc.StartUndo()

        # Export as a proxy
        export = export_selection_as_cproxy(doc, obj, proxy_path)
        if export != None:
            print("[Proxify] Proxy export successful")
            proxy = create_cproxy(doc, obj, proxy_path)
            doc.InsertObject(proxy)
            doc.AddUndo(c4d.UNDOTYPE_NEWOBJ, proxy)
        else:
            print("[Proxify] Proxy export failed")

        # Transfer texture tags from the objects to the proxy
        for i in GetAllNodesRecursively(obj):
            for t in extract_tags(doc, i, c4d.TextureTag):
                new_tag = t.GetClone()
                proxy.InsertTag(new_tag)
                doc.AddUndo(c4d.UNDOTYPE_NEWOBJ, new_tag)

        # Hide original selection
        doc.AddUndo(c4d.UNDO_CHANGE_SMALL, obj)
        obj.SetEditorMode(c4d.MODE_OFF)
        obj.SetRenderMode(c4d.MODE_OFF)

        # Add progress hint
        c4d.StatusClear()
        c4d.StatusSetText('Export completed') 

        doc.EndUndo()

    c4d.EventAdd()

if __name__=='__main__':
    main()